@@ -3,7 +3,7 @@ import sys, textwrap
 sys.path.insert(0, "..")
 from mercurial import demandimport; demandimport.enable()
 from mercurial.commands import table, globalopts
-from mercurial.i18n import gettext as _
+from mercurial.i18n import gettext, _
 from mercurial.help import helptable
 
 def get_desc(docstr):
@@ -36,13 +36,20 @@ def get_cmd(cmd):
     attr = table[cmd]
     cmds = cmd.lstrip("^").split("|")
 
-    d['synopsis'] = attr[2]
     d['cmd'] = cmds[0]
     d['aliases'] = cmd.split("|")[1:]
     d['desc'] = get_desc(attr[0].__doc__)
     d['opts'] = list(get_opts(attr[1]))
-    return d
 
+    s = 'hg ' + cmds[0]
+    if len(attr) > 2:
+        if not attr[2].startswith('hg'):
+            s += ' ' + attr[2]
+        else:
+            s = attr[2]
+    d['synopsis'] = s
+
+    return d
 
 def show_doc(ui):
     def bold(s, text=""):
@@ -59,46 +66,44 @@ def show_doc(ui):
     underlined(_("COMMANDS"))
     h = {}
     for c, attr in table.items():
-            f = c.split("|")[0]
-            f = f.lstrip("^")
-            h[f] = c
+        f = c.split("|")[0]
+        f = f.lstrip("^")
+        h[f] = c
     cmds = h.keys()
     cmds.sort()
 
     for f in cmds:
-            if f.startswith("debug"): continue
-            d = get_cmd(h[f])
-            # synopsis
-            ui.write("%s::\n" % d['synopsis'].replace("hg ","", 1))
-            # description
-            ui.write("%s\n\n" % d['desc'][1])
-            # options
-            opt_output = list(d['opts'])
-            if opt_output:
-                opts_len = max([len(line[0]) for line in opt_output])
-                ui.write(_("    options:\n"))
-                for optstr, desc in opt_output:
-                    if desc:
-                        s = "%-*s  %s" % (opts_len, optstr, desc)
-                    else:
-                        s = optstr
-                    s = textwrap.fill(s, initial_indent=4 * " ",
-                                      subsequent_indent=(6 + opts_len) * " ")
-                    ui.write("%s\n" % s)
-                ui.write("\n")
-            # aliases
-            if d['aliases']:
-                ui.write(_("    aliases: %s\n\n") % " ".join(d['aliases']))
+        if f.startswith("debug"): continue
+        d = get_cmd(h[f])
+        # synopsis
+        ui.write("[[%s]]\n" % d['cmd'])
+        ui.write("%s::\n" % d['synopsis'].replace("hg ","", 1))
+        # description
+        ui.write("%s\n\n" % d['desc'][1])
+        # options
+        opt_output = list(d['opts'])
+        if opt_output:
+            opts_len = max([len(line[0]) for line in opt_output])
+            ui.write(_("    options:\n"))
+            for optstr, desc in opt_output:
+                if desc:
+                    s = "%-*s  %s" % (opts_len, optstr, desc)
+                else:
+                    s = optstr
+                s = textwrap.fill(s, initial_indent=4 * " ",
+                                  subsequent_indent=(6 + opts_len) * " ")
+                ui.write("%s\n" % s)
+            ui.write("\n")
+        # aliases
+        if d['aliases']:
+            ui.write(_("    aliases: %s\n\n") % " ".join(d['aliases']))
 
     # print topics
-    for t in helptable:
-        l = t.split("|")
-        section = l[-1]
-        underlined(_(section).upper())
-        doc = helptable[t]
+    for names, section, doc in helptable:
+        underlined(gettext(section).upper())
         if callable(doc):
             doc = doc()
-        ui.write(_(doc))
+        ui.write(gettext(doc))
         ui.write("\n")
 
 if __name__ == "__main__":
