@@ -2,27 +2,30 @@
 #
 # Copyright 2006 Vadim Gelfer <vadim.gelfer@gmail.com>
 #
-# This software may be used and distributed according to the terms
-# of the GNU General Public License, incorporated herein by reference.
-'''pulling, updating and merging in one command'''
+# This software may be used and distributed according to the terms of the
+# GNU General Public License version 2, incorporated herein by reference.
+
+'''pull, update and merge in one command'''
 
 from mercurial.i18n import _
 from mercurial.node import nullid, short
-from mercurial import commands, cmdutil, hg, util, url
+from mercurial import commands, cmdutil, hg, util, url, error
+from mercurial.lock import release
 
 def fetch(ui, repo, source='default', **opts):
-    '''Pull changes from a remote repository, merge new changes if needed.
+    '''pull changes from a remote repository, merge new changes if needed.
 
     This finds all changes from the repository at the specified path
     or URL and adds them to the local repository.
 
-    If the pulled changes add a new branch head, the head is automatically
-    merged, and the result of the merge is committed.  Otherwise, the
-    working directory is updated to include the new changes.
+    If the pulled changes add a new branch head, the head is
+    automatically merged, and the result of the merge is committed.
+    Otherwise, the working directory is updated to include the new
+    changes.
 
     When a merge occurs, the newly pulled changes are assumed to be
-    "authoritative".  The head of the new changes is used as the first
-    parent, with local changes as the second.  To switch the merge
+    "authoritative". The head of the new changes is used as the first
+    parent, with local changes as the second. To switch the merge
     order, use --switch-parent.
 
     See 'hg help dates' for a list of formats valid for -d/--date.
@@ -52,22 +55,24 @@ def fetch(ui, repo, source='default', **opts):
             raise util.Abort(_('outstanding uncommitted changes'))
         if del_:
             raise util.Abort(_('working directory is missing some files'))
-        if len(repo.branchheads(branch)) > 1:
+        bheads = repo.branchheads(branch)
+        bheads = [head for head in bheads if len(repo[head].children()) == 0]
+        if len(bheads) > 1:
             raise util.Abort(_('multiple heads in this branch '
                                '(use "hg heads ." and "hg merge" to merge)'))
 
-        cmdutil.setremoteconfig(ui, opts)
-
-        other = hg.repository(ui, ui.expandpath(source))
+        other = hg.repository(cmdutil.remoteui(repo, opts),
+                              ui.expandpath(source))
         ui.status(_('pulling from %s\n') %
                   url.hidepassword(ui.expandpath(source)))
         revs = None
         if opts['rev']:
-            if not other.local():
-                raise util.Abort(_("fetch -r doesn't work for remote "
-                                   "repositories yet"))
-            else:
+            try:
                 revs = [other.lookup(rev) for rev in opts['rev']]
+            except error.CapabilityError:
+                err = _("Other repository doesn't support revision lookup, "
+                        "so a rev cannot be specified.")
+                raise util.Abort(err)
 
         # Are there any changes at all?
         modheads = repo.pull(other, heads=revs)
@@ -76,6 +81,7 @@ def fetch(ui, repo, source='default', **opts):
 
         # Is this a simple fast-forward along the current branch?
         newheads = repo.branchheads(branch)
+        newheads = [head for head in newheads if len(repo[head].children()) == 0]
         newchildren = repo.changelog.nodesbetween([parent], newheads)[2]
         if len(newheads) == 1:
             if newchildren[0] != parent:
@@ -115,20 +121,20 @@ def fetch(ui, repo, source='default', **opts):
             err = hg.merge(repo, secondparent, remind=False)
 
         if not err:
-            mod, add, rem = repo.status()[:3]
             message = (cmdutil.logmessage(opts) or
                        (_('Automated merge with %s') %
                         url.removeauth(other.url())))
-            force_editor = opts.get('force_editor') or opts.get('edit')
-            n = repo.commit(mod + add + rem, message,
-                            opts['user'], opts['date'], force=True,
-                            force_editor=force_editor)
+            editor = cmdutil.commiteditor
+            if opts.get('force_editor') or opts.get('edit'):
+                editor = cmdutil.commitforceeditor
+            n = repo.commit(message, opts['user'], opts['date'],
+                            force=True, editor=editor)
             ui.status(_('new changeset %d:%s merges remote changes '
                         'with local\n') % (repo.changelog.rev(n),
                                            short(n)))
 
     finally:
-        del lock, wlock
+        release(lock, wlock)
 
 cmdtable = {
     'fetch':
