@@ -2,28 +2,28 @@
 #
 # Copyright 2006, 2007 Brendan Cully <brendan@kublai.com>
 #
-# This software may be used and distributed according to the terms
-# of the GNU General Public License, incorporated herein by reference.
+# This software may be used and distributed according to the terms of the
+# GNU General Public License version 2, incorporated herein by reference.
 
-from mercurial.i18n import _
-import os, tempfile
-from mercurial import bundlerepo, changegroup, cmdutil, hg, merge
-from mercurial import patch, revlog, util
-
-'''patch transplanting tool
+'''command to transplant changesets from another branch
 
 This extension allows you to transplant patches from another branch.
 
-Transplanted patches are recorded in .hg/transplant/transplants, as a map
-from a changeset hash to its hash in the source repository.
+Transplanted patches are recorded in .hg/transplant/transplants, as a
+map from a changeset hash to its hash in the source repository.
 '''
 
-class transplantentry:
+from mercurial.i18n import _
+import os, tempfile
+from mercurial import bundlerepo, changegroup, cmdutil, hg, merge, match
+from mercurial import patch, revlog, util, error
+
+class transplantentry(object):
     def __init__(self, lnode, rnode):
         self.lnode = lnode
         self.rnode = rnode
 
-class transplants:
+class transplants(object):
     def __init__(self, path=None, transplantfile=None, opener=None):
         self.path = path
         self.transplantfile = transplantfile
@@ -64,12 +64,13 @@ class transplants:
         del self.transplants[self.transplants.index(transplant)]
         self.dirty = True
 
-class transplanter:
+class transplanter(object):
     def __init__(self, ui, repo):
         self.ui = ui
         self.path = repo.join('transplant')
         self.opener = util.opener(self.path)
-        self.transplants = transplants(self.path, 'transplants', opener=self.opener)
+        self.transplants = transplants(self.path, 'transplants',
+                                       opener=self.opener)
 
     def applied(self, repo, node, parent):
         '''returns True if a node is already an ancestor of parent
@@ -88,7 +89,7 @@ class transplanter:
 
     def apply(self, repo, source, revmap, merges, opts={}):
         '''apply the revisions in revmap one by one in revision order'''
-        revs = util.sort(revmap)
+        revs = sorted(revmap)
         p1, p2 = repo.dirstate.parents()
         pulls = []
         diffopts = patch.diffopts(self.ui, opts)
@@ -109,8 +110,8 @@ class transplanter:
 
                 parents = source.changelog.parents(node)
                 if not opts.get('filter'):
-                    # If the changeset parent is the same as the wdir's parent,
-                    # just pull it.
+                    # If the changeset parent is the same as the
+                    # wdir's parent, just pull it.
                     if parents[0] == p1:
                         pulls.append(node)
                         p1 = node
@@ -124,9 +125,9 @@ class transplanter:
 
                 domerge = False
                 if node in merges:
-                    # pulling all the merge revs at once would mean we couldn't
-                    # transplant after the latest even if transplants before them
-                    # fail.
+                    # pulling all the merge revs at once would mean we
+                    # couldn't transplant after the latest even if
+                    # transplants before them fail.
                     domerge = True
                     if not hasnode(repo, node):
                         repo.pull(source, heads=[node])
@@ -155,8 +156,9 @@ class transplanter:
                             self.ui.status(_('%s merged at %s\n') % (revstr,
                                       revlog.short(n)))
                         elif n:
-                            self.ui.status(_('%s transplanted to %s\n') % (revlog.short(node),
-                                                                       revlog.short(n)))
+                            self.ui.status(_('%s transplanted to %s\n')
+                                           % (revlog.short(node),
+                                              revlog.short(n)))
                     finally:
                         if patchfile:
                             os.unlink(patchfile)
@@ -166,7 +168,8 @@ class transplanter:
         finally:
             self.saveseries(revmap, merges)
             self.transplants.write()
-            del lock, wlock
+            lock.release()
+            wlock.release()
 
     def filter(self, filter, changelog, patchfile):
         '''arbitrarily rewrite changeset before applying it'''
@@ -214,10 +217,11 @@ class transplanter:
             try:
                 files = {}
                 try:
-                    fuzz = patch.patch(patchfile, self.ui, cwd=repo.root,
-                                       files=files)
+                    patch.patch(patchfile, self.ui, cwd=repo.root,
+                                files=files, eolmode=None)
                     if not files:
-                        self.ui.warn(_('%s: empty changeset') % revlog.hex(node))
+                        self.ui.warn(_('%s: empty changeset')
+                                     % revlog.hex(node))
                         return None
                 finally:
                     files = patch.updatedir(self.ui, repo, files)
@@ -231,14 +235,18 @@ class transplanter:
                 p2 = node
                 self.log(user, date, message, p1, p2, merge=merge)
                 self.ui.write(str(inst) + '\n')
-                raise util.Abort(_('Fix up the merge and run hg transplant --continue'))
+                raise util.Abort(_('Fix up the merge and run '
+                                   'hg transplant --continue'))
         else:
             files = None
         if merge:
             p1, p2 = repo.dirstate.parents()
             repo.dirstate.setparents(p1, node)
+            m = match.always(repo.root, '')
+        else:
+            m = match.exact(repo.root, '', files)
 
-        n = repo.commit(files, message, user, date, extra=extra)
+        n = repo.commit(message, user, date, extra=extra, match=m)
         if not merge:
             self.transplants.set(n, node)
 
@@ -280,7 +288,7 @@ class transplanter:
                                  revlog.hex(parents[0]))
             if merge:
                 repo.dirstate.setparents(p1, parents[1])
-            n = repo.commit(None, message, user, date, extra=extra)
+            n = repo.commit(message, user, date, extra=extra)
             if not n:
                 raise util.Abort(_('commit failed'))
             if not merge:
@@ -289,7 +297,7 @@ class transplanter:
 
             return n, node
         finally:
-            del wlock
+            wlock.release()
 
     def readseries(self):
         nodes = []
@@ -310,7 +318,7 @@ class transplanter:
         if not os.path.isdir(self.path):
             os.mkdir(self.path)
         series = self.opener('series', 'w')
-        for rev in util.sort(revmap):
+        for rev in sorted(revmap):
             series.write(revlog.hex(revmap[rev]) + '\n')
         if merges:
             series.write('# Merges\n')
@@ -380,7 +388,7 @@ class transplanter:
 def hasnode(repo, node):
     try:
         return repo.changelog.rev(node) != None
-    except revlog.RevlogError:
+    except error.RevlogError:
         return False
 
 def browserevs(ui, repo, nodes, opts):
@@ -408,7 +416,7 @@ def browserevs(ui, repo, nodes, opts):
             elif action == 'p':
                 parent = repo.changelog.parents(node)[0]
                 for chunk in patch.diff(repo, parent, node):
-                    repo.ui.write(chunk)
+                    ui.write(chunk)
                 action = None
             elif action not in ('y', 'n', 'm', 'c', 'q'):
                 ui.write('no such option\n')
@@ -435,28 +443,31 @@ def transplant(ui, repo, *revs, **opts):
     (transplanted from CHANGESETHASH)
 
     You can rewrite the changelog message with the --filter option.
-    Its argument will be invoked with the current changelog message
-    as $1 and the patch as $2.
+    Its argument will be invoked with the current changelog message as
+    $1 and the patch as $2.
 
-    If --source is specified, selects changesets from the named
-    repository. If --branch is specified, selects changesets from the
-    branch holding the named revision, up to that revision. If --all
-    is specified, all changesets on the branch will be transplanted,
-    otherwise you will be prompted to select the changesets you want.
+    If --source/-s is specified, selects changesets from the named
+    repository. If --branch/-b is specified, selects changesets from
+    the branch holding the named revision, up to that revision. If
+    --all/-a is specified, all changesets on the branch will be
+    transplanted, otherwise you will be prompted to select the
+    changesets you want.
 
-    hg transplant --branch REVISION --all will rebase the selected branch
-    (up to the named revision) onto your current working directory.
+    hg transplant --branch REVISION --all will rebase the selected
+    branch (up to the named revision) onto your current working
+    directory.
 
-    You can optionally mark selected transplanted changesets as
-    merge changesets. You will not be prompted to transplant any
-    ancestors of a merged transplant, and you can merge descendants
-    of them normally instead of transplanting them.
+    You can optionally mark selected transplanted changesets as merge
+    changesets. You will not be prompted to transplant any ancestors
+    of a merged transplant, and you can merge descendants of them
+    normally instead of transplanting them.
 
     If no merges or revisions are provided, hg transplant will start
     an interactive changeset browser.
 
-    If a changeset application fails, you can fix the merge by hand and
-    then resume where you left off by calling hg transplant --continue.
+    If a changeset application fails, you can fix the merge by hand
+    and then resume where you left off by calling hg transplant
+    --continue/-c.
     '''
     def getremotechanges(repo, url):
         sourcerepo = ui.expandpath(url)
@@ -496,16 +507,19 @@ def transplant(ui, repo, *revs, **opts):
     def checkopts(opts, revs):
         if opts.get('continue'):
             if filter(lambda opt: opts.get(opt), ('branch', 'all', 'merge')):
-                raise util.Abort(_('--continue is incompatible with branch, all or merge'))
+                raise util.Abort(_('--continue is incompatible with '
+                                   'branch, all or merge'))
             return
         if not (opts.get('source') or revs or
                 opts.get('merge') or opts.get('branch')):
-            raise util.Abort(_('no source URL, branch tag or revision list provided'))
+            raise util.Abort(_('no source URL, branch tag or revision '
+                               'list provided'))
         if opts.get('all'):
             if not opts.get('branch'):
                 raise util.Abort(_('--all requires a branch revision'))
             if revs:
-                raise util.Abort(_('--all is incompatible with a revision list'))
+                raise util.Abort(_('--all is incompatible with a '
+                                   'revision list'))
 
     checkopts(opts, revs)
 
@@ -517,7 +531,7 @@ def transplant(ui, repo, *revs, **opts):
     tp = transplanter(ui, repo)
 
     p1, p2 = repo.dirstate.parents()
-    if p1 == revlog.nullid:
+    if len(repo) > 0 and p1 == revlog.nullid:
         raise util.Abort(_('no revision checked out'))
     if not opts.get('continue'):
         if p2 != revlog.nullid:
@@ -553,9 +567,11 @@ def transplant(ui, repo, *revs, **opts):
                 revmap[int(r)] = source.lookup(r)
         elif opts.get('all') or not merges:
             if source != repo:
-                alltransplants = incwalk(source, incoming, branches, match=matchfn)
+                alltransplants = incwalk(source, incoming, branches,
+                                         match=matchfn)
             else:
-                alltransplants = transplantwalk(source, p1, branches, match=matchfn)
+                alltransplants = transplantwalk(source, p1, branches,
+                                                match=matchfn)
             if opts.get('all'):
                 revs = alltransplants
             else:
@@ -581,7 +597,9 @@ cmdtable = {
           ('p', 'prune', [], _('skip over REV')),
           ('m', 'merge', [], _('merge at REV')),
           ('', 'log', None, _('append transplant info to log message')),
-          ('c', 'continue', None, _('continue last transplant session after repair')),
+          ('c', 'continue', None, _('continue last transplant session '
+                                    'after repair')),
           ('', 'filter', '', _('filter changesets through FILTER'))],
-         _('hg transplant [-s REPOSITORY] [-b BRANCH [-a]] [-p REV] [-m REV] [REV]...'))
+         _('hg transplant [-s REPOSITORY] [-b BRANCH [-a]] [-p REV] '
+           '[-m REV] [REV]...'))
 }
