@@ -2,21 +2,15 @@
 #
 # Copyright 2006, 2007 Matt Mackall <mpm@selenic.com>
 #
-# This software may be used and distributed according to the terms
-# of the GNU General Public License, incorporated herein by reference.
+# This software may be used and distributed according to the terms of the
+# GNU General Public License version 2, incorporated herein by reference.
 
 from node import nullid, nullrev, short, hex
 from i18n import _
-import ancestor, bdiff, revlog, util, os, errno
+import ancestor, bdiff, error, util, subrepo
+import os, errno
 
-class propertycache(object):
-    def __init__(self, func):
-        self.func = func
-        self.name = func.__name__
-    def __get__(self, obj, type=None):
-        result = self.func(obj)
-        setattr(obj, self.name, result)
-        return result
+propertycache = util.propertycache
 
 class changectx(object):
     """A changecontext object makes access to data related to a particular
@@ -60,24 +54,28 @@ class changectx(object):
     def __nonzero__(self):
         return self._rev != nullrev
 
+    @propertycache
     def _changeset(self):
         return self._repo.changelog.read(self.node())
-    _changeset = propertycache(_changeset)
 
+    @propertycache
     def _manifest(self):
         return self._repo.manifest.read(self._changeset[0])
-    _manifest = propertycache(_manifest)
 
+    @propertycache
     def _manifestdelta(self):
         return self._repo.manifest.readdelta(self._changeset[0])
-    _manifestdelta = propertycache(_manifestdelta)
 
+    @propertycache
     def _parents(self):
         p = self._repo.changelog.parentrevs(self._rev)
         if p[1] == nullrev:
             p = p[:-1]
         return [changectx(self._repo, x) for x in p]
-    _parents = propertycache(_parents)
+
+    @propertycache
+    def substate(self):
+        return subrepo.state(self)
 
     def __contains__(self, key):
         return key in self._manifest
@@ -86,11 +84,12 @@ class changectx(object):
         return self.filectx(key)
 
     def __iter__(self):
-        for f in util.sort(self._manifest):
+        for f in sorted(self._manifest):
             yield f
 
     def changeset(self): return self._changeset
     def manifest(self): return self._manifest
+    def manifestnode(self): return self._changeset[0]
 
     def rev(self): return self._rev
     def node(self): return self._node
@@ -107,6 +106,14 @@ class changectx(object):
         """return contexts for each parent changeset"""
         return self._parents
 
+    def p1(self):
+        return self._parents[0]
+
+    def p2(self):
+        if len(self._parents) == 2:
+            return self._parents[1]
+        return changectx(self._repo, -1)
+
     def children(self):
         """return contexts for each child changeset"""
         c = self._repo.changelog.children(self._node)
@@ -125,15 +132,15 @@ class changectx(object):
             try:
                 return self._manifest[path], self._manifest.flags(path)
             except KeyError:
-                raise revlog.LookupError(self._node, path,
-                                         _('not found in manifest'))
+                raise error.LookupError(self._node, path,
+                                        _('not found in manifest'))
         if '_manifestdelta' in self.__dict__ or path in self.files():
             if path in self._manifestdelta:
                 return self._manifestdelta[path], self._manifestdelta.flags(path)
         node, flag = self._repo.manifest.find(self._changeset[0], path)
         if not node:
-            raise revlog.LookupError(self._node, path,
-                                     _('not found in manifest'))
+            raise error.LookupError(self._node, path,
+                                    _('not found in manifest'))
 
         return node, flag
 
@@ -143,7 +150,7 @@ class changectx(object):
     def flags(self, path):
         try:
             return self._fileinfo(path)[1]
-        except revlog.LookupError:
+        except error.LookupError:
             return ''
 
     def filectx(self, path, fileid=None, filelog=None):
@@ -161,22 +168,25 @@ class changectx(object):
         return changectx(self._repo, n)
 
     def walk(self, match):
-        fdict = dict.fromkeys(match.files())
+        fset = set(match.files())
         # for dirstate.walk, files=['.'] means "walk the whole tree".
         # follow that here, too
-        fdict.pop('.', None)
+        fset.discard('.')
         for fn in self:
-            for ffn in fdict:
+            for ffn in fset:
                 # match if the file is the exact name or a directory
                 if ffn == fn or fn.startswith("%s/" % ffn):
-                    del fdict[ffn]
+                    fset.remove(ffn)
                     break
             if match(fn):
                 yield fn
-        for fn in util.sort(fdict):
+        for fn in sorted(fset):
             if match.bad(fn, 'No such file in rev ' + str(self)) and match(fn):
                 yield fn
 
+    def sub(self, path):
+        return subrepo.subrepo(self, path)
+
 class filectx(object):
     """A filecontext object makes access to data related to a particular
        filerevision convenient."""
@@ -201,41 +211,41 @@ class filectx(object):
         if fileid is not None:
             self._fileid = fileid
 
+    @propertycache
     def _changectx(self):
         return changectx(self._repo, self._changeid)
-    _changectx = propertycache(_changectx)
 
+    @propertycache
     def _filelog(self):
         return self._repo.file(self._path)
-    _filelog = propertycache(_filelog)
 
+    @propertycache
     def _changeid(self):
         if '_changectx' in self.__dict__:
             return self._changectx.rev()
         else:
             return self._filelog.linkrev(self._filerev)
-    _changeid = propertycache(_changeid)
 
+    @propertycache
     def _filenode(self):
         if '_fileid' in self.__dict__:
             return self._filelog.lookup(self._fileid)
         else:
             return self._changectx.filenode(self._path)
-    _filenode = propertycache(_filenode)
 
+    @propertycache
     def _filerev(self):
         return self._filelog.rev(self._filenode)
-    _filerev = propertycache(_filerev)
 
+    @propertycache
     def _repopath(self):
         return self._path
-    _repopath = propertycache(_repopath)
 
     def __nonzero__(self):
         try:
-            n = self._filenode
+            self._filenode
             return True
-        except revlog.LookupError:
+        except error.LookupError:
             # file is missing
             return False
 
@@ -316,7 +326,7 @@ class filectx(object):
             try:
                 if fnode == p.filenode(name):
                     return None
-            except revlog.LookupError:
+            except error.LookupError:
                 pass
         return renamed
 
@@ -419,7 +429,7 @@ class filectx(object):
             visit.extend(fn)
 
         hist = {}
-        for r, f in util.sort(visit):
+        for r, f in sorted(visit):
             curr = decorate(f.data(), f)
             for p in parents(f):
                 if p != nullid:
@@ -441,7 +451,7 @@ class filectx(object):
 
         # prime the ancestor cache for the working directory
         for c in (self, fc2):
-            if c._filerev == None:
+            if c._filerev is None:
                 pl = [(n.path(), n.filenode()) for n in c.parents()]
                 acache[(c._path, None)] = pl
 
@@ -513,8 +523,9 @@ class workingctx(changectx):
         return True
 
     def __contains__(self, key):
-        return self._dirstate[key] not in "?r"
+        return self._repo.dirstate[key] not in "?r"
 
+    @propertycache
     def _manifest(self):
         """generate a manifest corresponding to the working directory"""
 
@@ -536,27 +547,26 @@ class workingctx(changectx):
                 del man[f]
 
         return man
-    _manifest = propertycache(_manifest)
 
+    @propertycache
     def _status(self):
         return self._repo.status(unknown=True)
-    _status = propertycache(_status)
 
+    @propertycache
     def _user(self):
         return self._repo.ui.username()
-    _user = propertycache(_user)
 
+    @propertycache
     def _date(self):
         return util.makedate()
-    _date = propertycache(_date)
 
+    @propertycache
     def _parents(self):
         p = self._repo.dirstate.parents()
         if p[1] == nullid:
             p = p[:-1]
         self._parents = [changectx(self._repo, x) for x in p]
         return self._parents
-    _parents = propertycache(_parents)
 
     def manifest(self): return self._manifest
 
@@ -564,7 +574,7 @@ class workingctx(changectx):
     def date(self): return self._date
     def description(self): return self._text
     def files(self):
-        return util.sort(self._status[0] + self._status[1] + self._status[2])
+        return sorted(self._status[0] + self._status[1] + self._status[2])
 
     def modified(self): return self._status[0]
     def added(self): return self._status[1]
@@ -613,7 +623,14 @@ class workingctx(changectx):
         return self._parents[0].ancestor(c2) # punt on two parents for now
 
     def walk(self, match):
-        return util.sort(self._repo.dirstate.walk(match, True, False).keys())
+        return sorted(self._repo.dirstate.walk(match, True, False))
+
+    def dirty(self, missing=False):
+        "check whether a working directory is modified"
+
+        return (self.p2() or self.branch() != self.p1().branch() or
+                self.modified() or self.added() or self.removed() or
+                (missing and self.deleted()))
 
 class workingfilectx(filectx):
     """A workingfilectx object makes access to data related to a particular
@@ -631,17 +648,9 @@ class workingfilectx(filectx):
         if workingctx:
             self._changectx = workingctx
 
+    @propertycache
     def _changectx(self):
         return workingctx(self._repo)
-    _changectx = propertycache(_changectx)
-
-    def _repopath(self):
-        return self._repo.dirstate.copied(self._path) or self._path
-    _repopath = propertycache(_repopath)
-
-    def _filelog(self):
-        return self._repo.file(self._repopath)
-    _filelog = propertycache(_filelog)
 
     def __nonzero__(self):
         return True
@@ -649,35 +658,30 @@ class workingfilectx(filectx):
     def __str__(self):
         return "%s@%s" % (self.path(), self._changectx)
 
-    def filectx(self, fileid):
-        '''opens an arbitrary revision of the file without
-        opening a new filelog'''
-        return filectx(self._repo, self._repopath, fileid=fileid,
-                       filelog=self._filelog)
-
-    def rev(self):
-        if '_changectx' in self.__dict__:
-            return self._changectx.rev()
-        return self._filelog.linkrev(self._filerev)
-
     def data(self): return self._repo.wread(self._path)
     def renamed(self):
-        rp = self._repopath
-        if rp == self._path:
+        rp = self._repo.dirstate.copied(self._path)
+        if not rp:
             return None
         return rp, self._changectx._parents[0]._manifest.get(rp, nullid)
 
     def parents(self):
         '''return parent filectxs, following copies if necessary'''
-        p = self._path
-        rp = self._repopath
-        pcl = self._changectx._parents
+        def filenode(ctx, path):
+            return ctx._manifest.get(path, nullid)
+
+        path = self._path
         fl = self._filelog
-        pl = [(rp, pcl[0]._manifest.get(rp, nullid), fl)]
-        if len(pcl) > 1:
-            if rp != p:
-                fl = None
-            pl.append((p, pcl[1]._manifest.get(p, nullid), fl))
+        pcl = self._changectx._parents
+        renamed = self.renamed()
+
+        if renamed:
+            pl = [renamed + (None,)]
+        else:
+            pl = [(path, filenode(pcl[0], path), fl)]
+
+        for pc in pcl[1:]:
+            pl.append((path, filenode(pc, path), fl))
 
         return [filectx(self._repo, p, fileid=n, filelog=l)
                 for p,n,l in pl if n != nullid]
@@ -734,7 +738,7 @@ class memctx(object):
         parents = [(p or nullid) for p in parents]
         p1, p2 = parents
         self._parents = [changectx(self._repo, p) for p in (p1, p2)]
-        files = util.sort(util.unique(files))
+        files = sorted(set(files))
         self._status = [files, [], [], [], []]
         self._filectxfn = filectxfn
 
@@ -753,6 +757,12 @@ class memctx(object):
     def __nonzero__(self):
         return True
 
+    def __getitem__(self, key):
+        return self.filectx(key)
+
+    def p1(self): return self._parents[0]
+    def p2(self): return self._parents[1]
+
     def user(self): return self._user or self._repo.ui.username()
     def date(self): return self._date
     def description(self): return self._text
@@ -803,4 +813,3 @@ class memfilectx(object):
     def isexec(self): return 'x' in self._flags
     def islink(self): return 'l' in self._flags
     def renamed(self): return self._copied
-
