@@ -3,14 +3,14 @@
 # Copyright 2005, 2006 Matt Mackall <mpm@selenic.com>
 # Copyright 2006 Vadim Gelfer <vadim.gelfer@gmail.com>
 #
-# This software may be used and distributed according to the terms
-# of the GNU General Public License, incorporated herein by reference.
+# This software may be used and distributed according to the terms of the
+# GNU General Public License version 2, incorporated herein by reference.
 
 from node import bin, hex, nullid
 from i18n import _
-import repo, os, urllib, urllib2, urlparse, zlib, util, httplib
-import errno, socket, changegroup, statichttprepo
-import url
+import repo, changegroup, statichttprepo, error, url, util
+import os, urllib, urllib2, urlparse, zlib, httplib
+import errno, socket
 
 def zgenerator(f):
     zd = zlib.decompressobj()
@@ -39,6 +39,12 @@ class httprepository(repo.repository):
 
         self.urlopener = url.opener(ui, authinfo)
 
+    def __del__(self):
+        for h in self.urlopener.handlers:
+            h.close()
+            if hasattr(h, "close_all"):
+                h.close_all()
+
     def url(self):
         return self.path
 
@@ -47,9 +53,9 @@ class httprepository(repo.repository):
     def get_caps(self):
         if self.caps is None:
             try:
-                self.caps = util.set(self.do_read('capabilities').split())
-            except repo.RepoError:
-                self.caps = util.set()
+                self.caps = set(self.do_read('capabilities').split())
+            except error.RepoError:
+                self.caps = set()
             self.ui.debug(_('capabilities: %s\n') %
                           (' '.join(self.caps or ['none'])))
         return self.caps
@@ -77,7 +83,7 @@ class httprepository(repo.repository):
             raise
         except httplib.HTTPException, inst:
             self.ui.debug(_('http error while sending %s command\n') % cmd)
-            self.ui.print_exc()
+            self.ui.traceback()
             raise IOError(None, inst)
         except IndexError:
             # this only happens with Python 2.3, later versions raise URLError
@@ -94,24 +100,25 @@ class httprepository(repo.repository):
         except AttributeError:
             proto = resp.headers['content-type']
 
+        safeurl = url.hidepassword(self._url)
         # accept old "text/plain" and "application/hg-changegroup" for now
         if not (proto.startswith('application/mercurial-') or
                 proto.startswith('text/plain') or
                 proto.startswith('application/hg-changegroup')):
-            self.ui.debug(_("Requested URL: '%s'\n") % cu)
-            raise repo.RepoError(_("'%s' does not appear to be an hg repository")
-                               % self._url)
+            self.ui.debug(_("requested URL: '%s'\n") % url.hidepassword(cu))
+            raise error.RepoError(_("'%s' does not appear to be an hg repository")
+                                  % safeurl)
 
         if proto.startswith('application/mercurial-'):
             try:
                 version = proto.split('-', 1)[1]
                 version_info = tuple([int(n) for n in version.split('.')])
             except ValueError:
-                raise repo.RepoError(_("'%s' sent a broken Content-Type "
-                                     "header (%s)") % (self._url, proto))
+                raise error.RepoError(_("'%s' sent a broken Content-Type "
+                                        "header (%s)") % (safeurl, proto))
             if version_info > (0, 1):
-                raise repo.RepoError(_("'%s' uses newer protocol %s") %
-                                   (self._url, version))
+                raise error.RepoError(_("'%s' uses newer protocol %s") %
+                                      (safeurl, version))
 
         return resp
 
@@ -129,14 +136,27 @@ class httprepository(repo.repository):
         success, data = d[:-1].split(' ', 1)
         if int(success):
             return bin(data)
-        raise repo.RepoError(data)
+        raise error.RepoError(data)
 
     def heads(self):
         d = self.do_read("heads")
         try:
             return map(bin, d[:-1].split(" "))
         except:
-            raise util.UnexpectedOutput(_("unexpected response:"), d)
+            raise error.ResponseError(_("unexpected response:"), d)
+
+    def branchmap(self):
+        d = self.do_read("branchmap")
+        try:
+            branchmap = {}
+            for branchpart in d.splitlines():
+                branchheads = branchpart.split(' ')
+                branchname = urllib.unquote(branchheads[0])
+                branchheads = [bin(x) for x in branchheads[1:]]
+                branchmap[branchname] = branchheads
+            return branchmap
+        except:
+            raise error.ResponseError(_("unexpected response:"), d)
 
     def branches(self, nodes):
         n = " ".join(map(hex, nodes))
@@ -145,7 +165,7 @@ class httprepository(repo.repository):
             br = [ tuple(map(bin, b.split(" "))) for b in d.splitlines() ]
             return br
         except:
-            raise util.UnexpectedOutput(_("unexpected response:"), d)
+            raise error.ResponseError(_("unexpected response:"), d)
 
     def between(self, pairs):
         batch = 8 # avoid giant requests
@@ -156,7 +176,7 @@ class httprepository(repo.repository):
             try:
                 r += [ l and map(bin, l.split(" ")) or [] for l in d.splitlines() ]
             except:
-                raise util.UnexpectedOutput(_("unexpected response:"), d)
+                raise error.ResponseError(_("unexpected response:"), d)
         return r
 
     def changegroup(self, nodes, kind):
@@ -201,7 +221,7 @@ class httprepository(repo.repository):
                 try:
                     ret = int(resp_code)
                 except ValueError, err:
-                    raise util.UnexpectedOutput(
+                    raise error.ResponseError(
                             _('push failed (unexpected response):'), resp)
                 self.ui.write(output)
                 return ret
@@ -233,6 +253,6 @@ def instance(ui, path, create):
             inst = httprepository(ui, path)
         inst.between([(nullid, nullid)])
         return inst
-    except repo.RepoError:
+    except error.RepoError:
         ui.note('(falling back to static-http)\n')
         return statichttprepo.instance(ui, "static-" + path, create)
