@@ -3,16 +3,11 @@
 # Copyright 2005, 2006 Matt Mackall <mpm@selenic.com>
 # Copyright 2006 Vadim Gelfer <vadim.gelfer@gmail.com>
 #
-# This software may be used and distributed according to the terms
-# of the GNU General Public License, incorporated herein by reference.
+# This software may be used and distributed according to the terms of the
+# GNU General Public License version 2, incorporated herein by reference.
 
 from i18n import _
-
-class RepoError(Exception):
-    pass
-
-class NoCapability(RepoError):
-    pass
+import error
 
 class repository(object):
     def capable(self, name):
@@ -31,6 +26,18 @@ class repository(object):
     def requirecap(self, name, purpose):
         '''raise an exception if the given capability is not present'''
         if not self.capable(name):
-            raise NoCapability(_('cannot %s; remote repository does not '
-                                 'support the %r capability') %
-                               (purpose, name))
+            raise error.CapabilityError(
+                _('cannot %s; remote repository does not '
+                  'support the %r capability') % (purpose, name))
+
+    def local(self):
+        return False
+
+    def cancopy(self):
+        return self.local()
+
+    def rjoin(self, path):
+        url = self.url()
+        if url.endswith('/'):
+            return url + path
+        return url + '/' + path
